from typing import Any

SALT_CHARS = ...  # type: Any
DEFAULT_PBKDF2_ITERATIONS = ...  # type: Any

def pbkdf2_hex(data, salt, iterations=..., keylen=None, hashfunc=None): ...
def pbkdf2_bin(data, salt, iterations=..., keylen=None, hashfunc=None): ...
def safe_str_cmp(a, b): ...
def gen_salt(length): ...
def generate_password_hash(password, method='', salt_length=8): ...
def check_password_hash(pwhash, password): ...
def safe_join(directory, filename): ...
